import java.util.Map;

import uws.UWSException;

import uws.job.AbstractJob;
import uws.job.ErrorType;

/**
 * <p>Algorithm of Ackerman. It is a double recursive function whose the algorithm is:</p>
 * <pre>
 * 	if (m==0)
 * 		return n+1;
 * 	else{
 *		if (n==0)
 *			return A(m-1, 1);
 *		else
 *			return A(m-1, A(m, n-1)); 
 * 	}
 * </pre>
 */
public class JobAckerman extends AbstractJob {
	private static final long serialVersionUID = 1L;
	
	protected int m = 0;
	protected int n = 0;

	public JobAckerman(Map<String, String> lstParam) throws UWSException {
		super(lstParam);
	}

	@Override
	protected synchronized String generateJobId() {
		return "Ackerman_"+super.generateJobId();
	}

	@Override
	protected void jobWork() throws UWSException, InterruptedException {
		if (m < 0 || n < 0)
			throw new UWSException("Parameters incorrect: M and N must be BOTH >= 0 !");
		
		int result = ackerman(m, n);
		
		if (!thread.isInterrupted())
			additionalParameters.put("RESULT", "Ackerman("+m+", "+n+") = "+result);
		else
			throw new InterruptedException();
	}
	
	private synchronized int ackerman(int m, int n){
		if (thread.isInterrupted())
			return 0;
	
		if (m == 0)
			return n+1;
		else if (n == 0)
			return ackerman(m-1, 1);
		else
			return ackerman(m-1, ackerman(m, n-1));
	}

	@Override
	protected boolean loadAdditionalParams() throws UWSException {
		if (additionalParameters.containsKey("m")){
			try{
				m = Integer.parseInt(additionalParameters.get("m"));
				if (m < 0)
					throw new NumberFormatException();
			}catch(NumberFormatException nfe){
				throw new UWSException(UWSException.BAD_REQUEST, "The given M value (\""+additionalParameters.get("m")+"\") is incorrect: it must be a positive integer value !", ErrorType.TRANSIENT);
			}
		}
		if (additionalParameters.containsKey("n")){
			try{
				n = Integer.parseInt(additionalParameters.get("n"));
				if (n < 0)
					throw new NumberFormatException();
			}catch(NumberFormatException nfe){
				throw new UWSException(UWSException.BAD_REQUEST, "The given N value (\""+additionalParameters.get("n")+"\") is incorrect: it must be a positive integer value !", ErrorType.TRANSIENT);
			}
		}
		return true;
	}

}
