import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import uws.UWSException;
import uws.UWSToolBox;

import uws.job.AbstractJob;
import uws.job.ErrorType;
import uws.job.JobList;

import uws.job.serializer.UWSSerializer;

import uws.service.QueuedExtendedUWS;
import uws.service.UWSUrl;
import uws.service.UserIdentifier;


/**
 * <p>An extended UWS which manages an execution queue and is able to identify a user.
 * An "about" action is added so that to get some statistics about this UWS.</p>
 * 
 * @author Gr&eacute;gory Mantelet (CDS)
 * @version 02/2011
 * 
 * @see AboutAction
 */
public class MyExtendedUWS extends QueuedExtendedUWS {
	private static final long serialVersionUID = 1L;

	public MyExtendedUWS(int nbMaxRunningJobs) throws UWSException {
		super(nbMaxRunningJobs);
		
		// Set name:
		setName("Algorithms");
		
		// Set description:
		setDescription("This UWS aims to manage several JobLists. Each one manages one different type of Job. Jobs are \"famous\" algorithms which may take a long time in function of their parameters.");
		
		// Add a UWS action to get some statistics about this UWS:
		addUWSAction(0, new AboutAction<JobList<AbstractJob>, AbstractJob>(this));
		
		// Associate another MIME type with the XML serializer:
		if (hasSerializerFor(UWSSerializer.MIME_TYPE_XML))
			serializers.put("text/xml", getSerializer(UWSSerializer.MIME_TYPE_XML));
		
		// Add a user identification (by IP address):
		setUserIdentifier(new UserIdentifier() {
			private static final long serialVersionUID = 1L;

			@Override
			public String extractUserId(UWSUrl urlInterpreter, HttpServletRequest request) throws UWSException {
				return request.getRemoteAddr();
			}
		});
	}

	@Override
	public void sendError(UWSException error, HttpServletRequest request, HttpServletResponse response) throws IOException, UWSException {
		// Reset the whole response to ensure the output stream is free:
		if (response.isCommitted())
			return;
		response.reset();
		
		// If HTTP status code = 303 (see other), make a redirection:
		if (error.getHttpErrorCode() == UWSException.SEE_OTHER)
			redirect(error.getMessage(), request, response);
		
		// Else, display properly the exception:
		else{
			// Set the HTTP status code and the content type of the response:
			response.setStatus(error.getHttpErrorCode());
			response.setContentType(UWSSerializer.MIME_TYPE_HTML);
			
			PrintWriter out = response.getWriter();
			
			// Header:
			out.println("<html>\n\t<head>");
			out.println("\t\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />");
			out.println("\t\t<link href=\""+UWSToolBox.getServerResource("styles/uwstuto.css", request)+"\" rel=\"stylesheet\" type=\"text/css\" />");
			out.println("\t\t<title>UWS ERROR</title>");
			out.println("\t</head>\n\t<body>");
			
			// Title:
			String errorColor = (error.getUWSErrorType() == ErrorType.FATAL)?"red":"orange";
			out.println("\t\t<h1 style=\"text-align: center; background-color:"+errorColor+"; color: white; font-weight: bold;\">UWS ERROR - "+error.getHttpErrorCode()+"</h1>");
			
			// Description part:
			out.println("\t\t<h2>Description</h2>");
			out.println("\t\t<ul>");
			out.println("\t\t\t<li><b>Type: </b>"+error.getUWSErrorType()+"</li>");
			String msg = error.getMessage();
			int start=msg.indexOf("["), end=msg.indexOf("]");
			String context=null;
			if (start >= 0 && start < end){
				context = msg.substring(start+1, end);
				msg = msg.substring(end+1);
			}
			if (context != null)
				out.println("\t\t\t<li><b>Context: </b>"+context+"</li>");
			out.println("\t\t\t<li><b>Exception: </b>"+error.getClass().getName()+"</li>");
			out.println("\t\t\t<li><b>Message:</b><p>"+msg+"</p></li>");
			out.println("\t\t</ul>");
			
			// Stack trace part:
			out.println("\t\t<h2>Stack trace</h2>");
			out.println("\t\t<table style=\"width: ihnerit;\">");
			out.println("\t\t\t<tr><th>Class</th><th>Method</th><th>Line</th></tr>");
			StackTraceElement[] trace = error.getStackTrace();
			for(int i=0; i<trace.length; i++){
				String className = trace[i].getClassName();
				if (className.startsWith("uws."))
					className = "<a href=\"/uwstuto/javadoc/"+className.replaceAll("\\.", "/")+".html\" class=\"javadoc\">"+className+"</a>";
				out.println("\t\t\t<tr"+((i%2 != 0)?" class=\"alt\"":"")+"><td>"+className+"</td><td>"+trace[i].getMethodName()+"</td><td>"+trace[i].getLineNumber()+"</td></tr>");
			}
			out.println("\t\t</table>");
			out.println("\t</body>\n</html>");
			
			out.close();
		}
	}
	
}
