import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import uws.UWSException;

import uws.job.AbstractJob;
import uws.job.JobList;
import uws.job.serializer.UWSSerializer;

import uws.service.AbstractUWS;
import uws.service.UWSUrl;

import uws.job.manager.ExecutionManager;

import uws.service.actions.UWSAction;


public class AboutAction<JL extends JobList<J>, J extends AbstractJob> extends UWSAction<JL, J> {
	private static final long serialVersionUID = 1L;

	public AboutAction(AbstractUWS<JL, J> u) {
		super(u);
	}

	@Override
	public String getName() {
		return "About UWS";
	}

	@Override
	public String getDescription() {
		return "General information and statistics about the UWS Algorithms. (URL: {baseUWS_URL}, Method: HTTP-GET or HTTP-POST, Parameter: ACTION=ABOUT)";
	}

	@Override
	public boolean match(UWSUrl urlInterpreter, String userId, HttpServletRequest request) throws UWSException {
		return !urlInterpreter.hasJobList()										// no jobs list specified
				&& request.getParameter("ACTION") != null						// at least, 1 parameter = ACTION
				&& request.getParameter("ACTION").equalsIgnoreCase("about");	// its value must be ABOUT (not case sensitive)
	}

	@Override
	public boolean apply(UWSUrl urlInterpreter, String userId, HttpServletRequest request, HttpServletResponse response) throws UWSException, IOException {
		response.setContentType("text/html");
		
		PrintWriter out = response.getWriter();
		
		BufferedReader reader = new BufferedReader(new FileReader(request.getSession().getServletContext().getRealPath("/about_header.txt")));
		try{
			String line = null;
			while((line = reader.readLine()) != null)
				out.println(line);
		}finally { reader.close(); }
		
		out.println("<h1>About the UWS Algorithms</h1>");
		
		ExecutionManager manager = uws.getExecutionManager();
		
		out.println("<h2>UWS Algorithms</h2>");
		out.println("<ul>");
		out.println("<li><b>Name:</b> "+uws.getName()+"</li>");
		out.println("<li><b>Description:</b><p style=\"margin-bottom: 0; padding-bottom: 0\">"+uws.getDescription()+"</p></li>");
		out.println("<li><b>Base UWS URL:</b> "+uws.getBaseURI()+"</li><br />");
		out.println("<li><b>Nb Queued Jobs:</b> "+manager.getNbQueuedJobs()+"</li>");
		
		out.print("<li><b>Nb Running Jobs:</b> "+manager.getNbRunningJobs());
		if (manager.getNbRunningJobs() > 0){
			Iterator<AbstractJob> it = manager.getRunningJobs();
			String runningJobs = null;
			while(it.hasNext())
				runningJobs = ((runningJobs==null)?"":(runningJobs+", "))+it.next().getJobId();
			out.print(" ("+runningJobs+")");
		}
		out.println("</li>");
		
		out.println("<li><b>Nb Max Running Jobs:</b> 3</li>");
		out.println("<li><b>Nb Jobs Lists:</b> "+uws.getNbJobList()+"</li>");
		out.println("</ul>");
		
		out.println("<h2>"+uws.getNbSerializers()+" Available Serializers</h2>");
		out.println("<ul>");
		Iterator<UWSSerializer> itSerializers = uws.getSerializers();
		while(itSerializers.hasNext()){
			UWSSerializer serializer = itSerializers.next();
			out.println("<li><b>"+serializer.getMimeType()+(serializer.getMimeType().equalsIgnoreCase(uws.getDefaultSerializer())?" <i>(default)</i>":"")+"</b></li>");
		}
		out.println("</ul>");
		
		out.println("<h2>"+uws.getNbUWSActions()+" Available Actions</h2>");
		out.println("<ul>");
		Iterator<UWSAction<JL,J>> itActions = uws.getUWSActions();
		while(itActions.hasNext()){
			UWSAction<JL,J> action = itActions.next();
			out.println("<li><b>"+action.getName()+"</b><p style=\"margin-bottom: 0; padding-bottom: 0\"><i>"+action.getDescription()+"</i></p></li>");
		}
		out.println("</ul>");

		out.println("<h2>Jobs Lists</h2>");
		out.println("<table style=\"text-align: center; width: 100%;\">");
		out.println("<tr><th></th><th>Users</th><th>Jobs</th><th>Pending</th><th>Queued</th><th>Running</th><th>Complete</th><th>Error</th><th>Aborted</th><th>Others</th></tr>");
		for(JL jl : uws){
			int nbPending=0, nbQueued=0, nbRunning=0, nbComplete=0, nbError=0, nbAborted=0, nbOthers=0;
			for(J job : jl){
				switch(job.getPhase()){
					case PENDING: nbPending++; break;
					case QUEUED: nbQueued++; break;
					case EXECUTING: nbRunning++; break;
					case COMPLETED: nbComplete++; break;
					case ERROR: nbError++; break;
					case ABORTED: nbAborted++; break;
					default: nbOthers++; break;
				}
			}
			out.println("<tr><td><b><a href=\"extended/"+jl.getName()+"\">"+jl.getName()+"</a></b></td><td>"+(jl.getNbUsers()<=0?"-":jl.getNbUsers())+"</td><td>"+(jl.getNbJobs()<=0?"-":jl.getNbJobs())+"</td><td>"+(nbPending<=0?"-":nbPending)+"</td><td>"+(nbQueued<=0?"-":nbQueued)+"</td><td>"+(nbRunning<=0?"-":nbRunning)+"</td><td>"+(nbComplete<=0?"-":nbComplete)+"</td><td>"+(nbError<=0?"-":nbError)+"</td><td>"+(nbAborted<=0?"-":nbAborted)+"</td><td>"+(nbOthers<=0?"-":nbOthers)+"</td></tr>");
		}
		out.println("</table>");
		
		reader = new BufferedReader(new FileReader(request.getSession().getServletContext().getRealPath("about_footer.txt")));
		try{
			String line = null;
			while((line = reader.readLine()) != null)
				out.println(line);
		}finally { reader.close(); }
		
		out.close();
		
		return true;
	}

}
